@@ -10,11 +10,12 @@ import traceback
 
 import aspen
 import aspen.logging
-from aspen.utils import ascii_dammit
+from aspen import execution
+from aspen.hooks import Hooks
 from aspen.configuration import parse
 from aspen.configuration.exceptions import ConfigurationError
-from aspen.configuration.hooks import Hooks
 from aspen.configuration.options import OptionParser, DEFAULT
+from aspen.utils import ascii_dammit
 
 
 # Nicer defaultdict
@@ -26,11 +27,13 @@ NO_DEFAULT = object()
 class NicerDefaultDict(collections.defaultdict):
     """Subclass to support .default assignment.
     """
-    
+
     __default = ''
     def _get_default(self, name):
+        """property getter for default property"""
         return self.__default
     def _set_default(self, value):
+        """property setter for default property"""
         self.default_factory = lambda: value
         self.__default = value
     default = property(_get_default, _set_default)
@@ -38,19 +41,19 @@ class NicerDefaultDict(collections.defaultdict):
 
     def get(self, name, default=NO_DEFAULT):
         if default is NO_DEFAULT:
-            default = self.default 
+            default = self.default
         collections.defaultdict.get(self, name, default)
 
 
 # Defaults
 # ========
-# The from_unicode callable converts from unicode to whatever format is 
-# required by the variable, raising ValueError appropriately. Note that 
+# The from_unicode callable converts from unicode to whatever format is
+# required by the variable, raising ValueError appropriately. Note that
 # name is supposed to match the options in our optparser. I like it wet.
 
 KNOBS = \
     { 'configuration_scripts': (lambda: [], parse.list_)
-    , 'network_engine':     (u'cherrypy', parse.network_engine)
+    , 'network_engine':     (u'cheroot', parse.network_engine)
     , 'network_address':    ( ((u'0.0.0.0', 8080), socket.AF_INET)
                             , parse.network_address
                              )
@@ -65,7 +68,7 @@ KNOBS = \
     , 'charset_dynamic':    (u'UTF-8', parse.charset)
     , 'charset_static':     (None, parse.charset)
     , 'indices':            ( lambda: [u'index.html', u'index.json', u'index']
-                            , parse.list_ 
+                            , parse.list_
                              )
     , 'list_directories':   (False, parse.yes_no)
     , 'media_type_default': ('text/plain', parse.media_type)
@@ -83,8 +86,13 @@ class Configurable(object):
     """Mixin object for aggregating configuration from several sources.
     """
 
+    protected = False  # Set to True to require authentication for all
+                       # requests.
+
     @classmethod
     def from_argv(cls, argv):
+        """return a Configurable based on the passed-in arguments list
+        """
         configurable = cls()
         configurable.configure(argv)
         return configurable
@@ -99,7 +107,7 @@ class Configurable(object):
         if name_in_context:
             assert isinstance(flat, unicode) # sanity check
             name_in_context = " %s=%s" % (name_in_context, flat)
-        out = "  %-22s %-30s %-24s" 
+        out = "  %-22s %-30s %-24s"
         return out % (name, hydrated, context + name_in_context)
 
     def set(self, name, raw, from_unicode, context, name_in_context):
@@ -196,6 +204,24 @@ class Configurable(object):
         # Set some attributes.
         # ====================
 
+
+        def safe_getcwd(errorstr):
+            try:
+                # If the working directory no longer exists, then the following
+                # will raise OSError: [Errno 2] No such file or directory. I
+                # swear I've seen this under supervisor, though I don't have
+                # steps to reproduce. :-(  To get around this you specify a
+                # www_root explicitly, or you can use supervisor's cwd
+                # facility.
+
+                return os.getcwd()
+            except OSError, err:
+                if err.errno != errno.ENOENT:
+                    raise
+                raise ConfigurationError(errorstr)
+
+
+
         # LOGGING_THRESHOLD
         # -----------------
         # This is initially set to -1 and not 0 so that we can tell if the user
@@ -205,31 +231,8 @@ class Configurable(object):
             aspen.logging.LOGGING_THRESHOLD = self.logging_threshold
         # Now that we know the user's desires, we can log appropriately.
         aspen.log_dammit(os.linesep.join(msgs))
-       
-        # www_root
-        if self.www_root is None:
-            try:
 
-                # If the working directory no longer exists, then the following
-                # will raise OSError: [Errno 2] No such file or directory. I
-                # swear I've seen this under supervisor, though I don't have
-                # steps to reproduce. :-(  To get around this you specify a
-                # www_root explicitly, or you can use supervisor's cwd
-                # facility.
-
-                self.www_root = os.getcwd()
-            except OSError, err:
-                if err.errno != errno.ENOENT:
-                    raise
-                raise ConfigurationError("Could not get a current working "
-                                         "directory. You can specify "
-                                         "ASPEN_WWW_ROOT in the environment, "
-                                         "or --www_root on the command line.")
-
-        self.www_root = os.path.realpath(self.www_root)
-        os.chdir(self.www_root)
-
-        # project root 
+        # project root
         if self.project_root is None:
             aspen.log_dammit("project_root not configured (no template bases, "
                              "etc.).")
@@ -237,11 +240,14 @@ class Configurable(object):
         else:
             # canonicalize it
             if not os.path.isabs(self.project_root):
-                aspen.log_dammit("project_root is relative: '%s'." 
+                aspen.log_dammit("project_root is relative to CWD: '%s'."
                                  % self.project_root)
-                self.project_root = os.path.join( self.www_root
-                                                , self.project_root
-                                                 )
+                cwd = safe_getcwd("Could not get a current working "
+                                  "directory. You can specify "
+                                  "ASPEN_PROJECT_ROOT in the environment, "
+                                  "or --project_root on the command line.")
+                self.project_root = os.path.join(cwd, self.project_root)
+
             self.project_root = os.path.realpath(self.project_root)
             aspen.log_dammit("project_root set to %s." % self.project_root)
 
@@ -258,23 +264,35 @@ class Configurable(object):
             # PYTHONPATH
             sys.path.insert(0, self.project_root)
 
+        # www_root
+        if self.www_root is None:
+            self.www_root = safe_getcwd("Could not get a current working "
+                                         "directory. You can specify "
+                                         "ASPEN_WWW_ROOT in the environment, "
+                                         "or --www_root on the command line.")
+
+        self.www_root = os.path.realpath(self.www_root)
+        os.chdir(self.www_root)
+
         # renderers
         self.renderer_factories = {}
         for name in aspen.RENDERERS:
             # Pre-populate renderers so we can report on ImportErrors early
             try:
                 capture = {}
-                python_syntax = 'from aspen.renderers.%s_ import Factory' 
+                python_syntax = 'from aspen.renderers.%s import Factory'
                 exec python_syntax % name in capture
                 make_renderer = capture['Factory'](self)
             except ImportError, err:
                 make_renderer = err
+                err.info = sys.exc_info()
             self.renderer_factories[name] = make_renderer
 
         default_renderer = self.renderer_factories[self.renderer_default]
         if isinstance(default_renderer, ImportError):
-            msg = "\033[1;31mImportError loading the default renderer, %s:\033[0m" 
+            msg = "\033[1;31mImportError loading the default renderer, %s:\033[0m"
             aspen.log_dammit(msg % self.renderer_default)
+            sys.excepthook(*default_renderer.info)
             raise default_renderer
 
         aspen.log_dammit("Renderers (*ed are unavailable, CAPS is default):")
@@ -298,17 +316,17 @@ class Configurable(object):
         mimetypes.init()
 
         # network_engine
-        try: 
+        try:
             capture = {}
-            python_syntax = 'from aspen.network_engines.%s_ import Engine' 
+            python_syntax = 'from aspen.network_engines.%s_ import Engine'
             exec python_syntax % self.network_engine in capture
             Engine = capture['Engine']
         except ImportError:
-            # ANSI colors: 
+            # ANSI colors:
             #   http://stackoverflow.com/questions/287871/
             #   http://en.wikipedia.org/wiki/ANSI_escape_code#CSI_codes
             #   XXX consider http://pypi.python.org/pypi/colorama
-            msg = "\033[1;31mImportError loading the %s network engine:\033[0m" 
+            msg = "\033[1;31mImportError loading the %s network engine:\033[0m"
             aspen.log_dammit(msg % self.network_engine)
             raise
         self.network_engine = Engine(self.network_engine, self)
@@ -321,19 +339,31 @@ class Configurable(object):
             self.network_port = None
 
         # hooks
-        self.hooks = Hooks([ 'startup'
-                           , 'inbound_early'
-                           , 'inbound_late'
-                           , 'outbound_early'
-                           , 'outbound_late'
-                           , 'shutdown'
-                            ])
+        self.hooks = Hooks()
+        self.hooks.startup = []
+        self.hooks.inbound_early = []
+        self.hooks.inbound_core = []
+        self.hooks.inbound_late = []
+        self.hooks.error_early = []
+        self.hooks.error_late = []
+        self.hooks.outbound = []
+        self.hooks.shutdown = []
+
+
+        # Set up core logic.
+        # ==================
+        # This way, apps have fairly complete control over the request handling
+        # cycle. We don't have an error_core because error handling is more
+        # complicated.
+
+        self.reset_inbound_core()
+        self.reset_outbound()
 
 
         # Finally, exec any configuration scripts.
         # ========================================
         # The user gets self as 'website' inside their configuration scripts.
-       
+
         for filepath in self.configuration_scripts:
             if not filepath.startswith(os.sep):
                 if self.project_root is None:
@@ -345,8 +375,8 @@ class Configurable(object):
             try:
                 execfile(filepath, {'website': self})
             except IOError, err:
-                # I was checking os.path.isfile for these, but then we have a 
-                # race condition that smells to me like a potential security 
+                # I was checking os.path.isfile for these, but then we have a
+                # race condition that smells to me like a potential security
                 # vulnerability.
                 if err.errno == errno.ENOENT:
                     msg = ("The configuration script %s doesn't seem to "
@@ -366,3 +396,5 @@ class Configurable(object):
                     else:
                         raise ConfigurationError(msg % filepath)
                 # XXX smelly ... bug here? second else pls?
+            else:
+                execution.if_changes(filepath)
